// generated from rosidl_adapter/resource/msg.idl.em
// with input from rmw_dds_common/msg/Gid.msg
// generated code does not contain a copyright notice


module rmw_dds_common {
  module msg {
    typedef uint8 uint8__16[16];
    struct Gid {
      uint8__16 data;
    };
  };
};
